from dagster.core.storage.pipeline_run import PipelineRun
from dagster.core.test_utils import create_run_for_test, poll_for_finished_run
from dagster.utils import merge_dicts

from .utils import setup_instance, start_daemon


def create_run(instance, external_pipeline, **kwargs):  # pylint: disable=redefined-outer-name
    pipeline_args = merge_dicts(
        {
            "pipeline_name": "foo_pipeline",
            "external_pipeline_origin": external_pipeline.get_external_origin(),
        },
        kwargs,
    )
    return create_run_for_test(instance, **pipeline_args)


def assert_events_in_order(logs, expected_events):

    logged_events = [log.dagster_event.event_type_value for log in logs if log.is_dagster_event]
    filtered_logged_events = [event for event in logged_events if event in expected_events]

    assert filtered_logged_events == expected_events


def test_queue_from_schedule_and_sensor(tmpdir, foo_example_repo):
    dagster_home_path = tmpdir.strpath
    with setup_instance(
        dagster_home_path,
        """run_coordinator:
    module: dagster.core.run_coordinator
    class: QueuedRunCoordinator
    config:
        dequeue_interval_seconds: 1
    """,
    ) as instance:
        external_schedule = foo_example_repo.get_external_schedule("always_run_schedule")
        external_sensor = foo_example_repo.get_external_sensor("always_on_sensor")
        external_pipeline = foo_example_repo.get_full_external_pipeline("foo_pipeline")

        instance.start_schedule_and_update_storage_state(external_schedule)
        instance.start_sensor(external_sensor)

        with start_daemon(timeout=180):
            run = create_run(instance, external_pipeline)
            instance.submit_run(run.run_id, external_pipeline)

            runs = [
                poll_for_finished_run(instance, run.run_id),
                poll_for_finished_run(
                    instance, run_tags=PipelineRun.tags_for_sensor(external_sensor)
                ),
                poll_for_finished_run(
                    instance,
                    run_tags=PipelineRun.tags_for_schedule(external_schedule),
                    timeout=90,
                ),
            ]

            for run in runs:
                logs = instance.all_logs(run.run_id)
                assert_events_in_order(
                    logs,
                    [
                        "PIPELINE_ENQUEUED",
                        "PIPELINE_DEQUEUED",
                        "PIPELINE_STARTING",
                        "PIPELINE_START",
                        "PIPELINE_SUCCESS",
                    ],
                )


def test_queued_runs(tmpdir, foo_example_repo):
    dagster_home_path = tmpdir.strpath
    with setup_instance(
        dagster_home_path,
        """run_coordinator:
    module: dagster.core.run_coordinator
    class: QueuedRunCoordinator
    config:
        dequeue_interval_seconds: 1
    """,
    ) as instance:
        with start_daemon():
            external_pipeline = foo_example_repo.get_full_external_pipeline("foo_pipeline")

            run = create_run(instance, external_pipeline)

            instance.submit_run(run.run_id, external_pipeline)

            poll_for_finished_run(instance, run.run_id)

            logs = instance.all_logs(run.run_id)
            assert_events_in_order(
                logs,
                [
                    "PIPELINE_ENQUEUED",
                    "PIPELINE_DEQUEUED",
                    "PIPELINE_STARTING",
                    "PIPELINE_START",
                    "PIPELINE_SUCCESS",
                ],
            )



import contextlib
import os

from dagster import seven
from dagster.core.instance import DagsterInstance
from dagster.serdes.ipc import interrupt_ipc_subprocess, open_ipc_subprocess


@contextlib.contextmanager
def setup_instance(dagster_home, instance_config):
    os.environ["DAGSTER_HOME"] = dagster_home

    with open(os.path.join(dagster_home, "dagster.yaml"), "w") as file:
        file.write(instance_config)

    with DagsterInstance.get() as instance:
        yield instance


@contextlib.contextmanager
def start_daemon(timeout=30):
    p = open_ipc_subprocess(["dagster-daemon", "run"])
    try:
        yield
    finally:
        interrupt_ipc_subprocess(p)
        seven.wait_for_process(p, timeout=timeout)



from dagster import RunRequest, pipeline, repository, schedule, sensor, solid


@solid()
def foo_solid(_):
    pass


@pipeline
def foo_pipeline():
    foo_solid()


@pipeline
def other_foo_pipeline():
    foo_solid()


@schedule(
    pipeline_name="foo_pipeline",
    cron_schedule="*/1 * * * *",
)
def always_run_schedule(_context):
    return {}


@sensor(pipeline_name="foo_pipeline")
def always_on_sensor(_context):
    return RunRequest(run_key=None, run_config={}, tags={})


@repository
def example_repo():
    return [foo_pipeline, always_run_schedule, always_on_sensor]


@repository
def other_example_repo():
    return [other_foo_pipeline]



import logging
from contextlib import contextmanager

import pytest
from dagster.utils.log import define_structured_logger
from dagster.utils.test import create_test_pipeline_execution_context


@contextmanager
def construct_structured_logger(constructor=lambda x: x):
    messages = []

    def _append_message(logger_message):
        messages.append(constructor(logger_message))

    logger_def = define_structured_logger("some_name", _append_message, level=logging.DEBUG)
    yield logger_def, messages


def test_structured_logger_in_context():
    with construct_structured_logger() as (logger, messages):
        context = create_test_pipeline_execution_context(logger_defs={"structured_log": logger})
        context.log.debug("from_context", foo=2)
        assert len(messages) == 1
        message = messages[0]
        assert message.name == "some_name"
        assert message.level == logging.DEBUG
        assert message.meta["foo"] == 2
        assert message.meta["orig_message"] == "from_context"


def test_structured_logger_in_context_with_bad_log_level():
    messages = []

    def _append_message(logger_message):
        messages.append(logger_message)

    logger = define_structured_logger("some_name", _append_message, level=logging.DEBUG)
    context = create_test_pipeline_execution_context(logger_defs={"structured_logger": logger})
    with pytest.raises(AttributeError):
        context.log.gargle("from_context", foo=2)
